/* funkload doc css */
/* $Id: nuxeo_doc.css 21596 2005-02-15 10:01:33Z bdelbosc $ */

body {
  font: 90% 'Lucida Grande', Verdana, Geneva, Lucida, Arial, Helvetica, sans-serif;
  background: #ffffff;
  color: black;
  margin: 2em;
  padding: 2em;
}

a[href] {
  color: #436976;
  background-color: transparent;
}

h1 a[href] {
  text-decoration: none;
  color: #fcb100;
  background-color: transparent;
}

a.strong {
  font-weight: bold;
}

img {
  margin: 0;
  border: 0;
}

p {
  margin: 0.5em 0 1em 0;
  line-height: 1.5em;
}
p a {
  text-decoration: underline;
}
p a:visited {
  color: purple;
  background-color: transparent;
}
p a:active {
  color: red;
  background-color: transparent;
}
a:hover {
  text-decoration: none;
}
p img {
  border: 0;
  margin: 0;
}

h1, h2, h3, h4, h5, h6 {
  color: #003a6b;
  background-color: transparent;
  font: 100% 'Lucida Grande', Verdana, Geneva, Lucida, Arial, Helvetica, sans-serif;
  margin: 0;
  padding-top: 0.5em;
}

h1 {
  font-size: 160%;
  margin-bottom: 0.5em;
  border-bottom: 1px solid #888;
  margin-top: 1em;
}

h1.title {
  font-size: 200%;
}

h2 {
  font-size: 140%;
  margin-bottom: 0.5em;
}
h3 {
  font-size: 120%;
  margin-bottom: 0.5em;
}
h4 {
  font-size: 110%;
  font-weight: bold;
}
h5 {
  font-size: 100%;
  font-weight: bold;
}
h6 {
  font-size: 80%;
  font-weight: bold;
}

ul a, ol a {
  text-decoration: underline;
}

dt {
  font-weight: bold;
}
dt a {
  text-decoration: none;
}

dd {
  line-height: 1.5em;
  margin-bottom: 1em;
}

legend {
  background: #ffffff;
  padding: 0.5em;
}

form {
  margin: 0;
}


dl.form {
  margin: 0;
  padding: 1em;
}

dl.form dt {
  width: 30%;
  float: left;
  margin: 0;
  padding: 0 0.5em 0.5em 0;
  text-align: right;
}

input {
  font: 100% 'Lucida Grande', Verdana, Geneva, Lucida, Arial, Helvetica, sans-serif;
  color: black;
  background-color: white;
  vertical-align: middle;
}

abbr, acronym, .explain {
  color: black;
  background-color: transparent;
}

q, blockquote {
}

code, pre {
  font-family: monospace;
  font-size: 1.2em;
  display: block;
  padding: 10px;
  border: 1px solid #838183;
  background-color: #eee;
  color: #000;
  overflow: auto;
  margin: 0.5em 1em;
}

table.docutils {
  border: 1px solid black;
  font-size: 0.6em;
  border-spacing: 0;
  padding: 0;
  margin: 1em;
}

table.docutils td {
  border: 1px solid #CCC;
  text-align: right;
  padding: 0 0.3em;
  margin: 0;
}

table.docutils tr {
 padding: 0;
 margin: 0;
}

table.docutils th {
  border: 1px solid #CCC;
  border-bottom: 1px solid black;
  padding: 0.3em;
  font-weight: bold;
  margin: 0;
}

table.docutils td p, table.docutils th p {
  margin: 0;
  padding: 0;
}

table.docutils th.field-name {
  border: 1px solid #CCC;
}

div.abstract {
  padding: 0.5em;
  background-color: #eef;
  border: 1px solid #838183;
  margin: 0.5em 1em;
}

div.abstract {
  padding: 0.5em;
  background-color: #eef;
  border: 1px solid #838183;
  margin: 0.5em 1em;
}

div.warning, div.note {
  padding: 0.5em;
  background-color: #fee;
  border: 1px solid #838183;
  margin: 0.5em 1em;
}

div.warning p.admonition-title {
  color: red ;
  font-weight: bold ;
}

div.abstract p.topic-title {
  font-weight: bold
}


th.field-name {
  font-weight: bold;
  font-size: 1.5em;
}

td.field-body {
  font-size: 1.5em;
}

tr.field {
  border: none;
}


div.contents p.first a {
  font-size: 150%;
  border-bottom: 1px solid #888;
  color: #fcb100;
}

div.contents a  { text-decoration: none }

pre.address {
  display: inline;
  border: none;
  padding: 0;
  margin: 0;
}

